/*
   +----------------------------------------------------------------------+
   | PHP Version 5                                                        |
   +----------------------------------------------------------------------+
   | Copyright (c) 1997-2004 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.0 of the PHP license,       |
   | that is bundled with this package in the file LICENSE, and is        |
   | available through the world-wide-web at the following url:           |
   | http://www.php.net/license/3_0.txt.                                  |
   | If you did not receive a copy of the PHP license and are unable to   |
   | obtain it through the world-wide-web, please send a note to          |
   | license@php.net so we can mail you a copy immediately.               |
   +----------------------------------------------------------------------+
   | Author:  Sascha Schumann <sascha@schumann.cx>                        |
   +----------------------------------------------------------------------+
 */


/* $Id: incomplete_class.c,v 1.24 2004/04/07 08:06:09 stas Exp $ */

#include "php.h"
#include "basic_functions.h"
#include "php_incomplete_class.h"

#define INCOMPLETE_CLASS_MSG \
		"The script tried to execute a method or "  \
		"access a property of an incomplete object. " \
		"Please ensure that the class definition <b>%s</b> of the object " \
		"you are trying to operate on was loaded _before_ " \
		"the session was started"


static zend_object_handlers php_incomplete_object_handlers;

/* {{{ incomplete_class_message
 */
static void incomplete_class_message(int error_type TSRMLS_DC)
{
	char buf[1024];
	char *class_name = NULL;

	if(EG(This)) {
		class_name = php_lookup_class_name(EG(This), NULL);
	}
	
	if (!class_name) {
		class_name = estrndup("unknown", sizeof("unknown")-1);
	}
	
	snprintf(buf, sizeof(buf)-1, INCOMPLETE_CLASS_MSG, class_name);
	
	efree(class_name);

	php_error_docref(NULL TSRMLS_CC, error_type, "%s", buf);
}
/* }}} */

static zval *incomplete_class_get_property(zval *object, zval *member, int type TSRMLS_DC)
{
	incomplete_class_message(E_NOTICE TSRMLS_CC);
	if(type == BP_VAR_W || type == BP_VAR_RW) {
		return EG(error_zval_ptr);
	} else {
		return EG(uninitialized_zval_ptr);
	}
}

static void incomplete_class_write_property(zval *object, zval *member, zval *value TSRMLS_DC)
{
	incomplete_class_message(E_NOTICE TSRMLS_CC);
}
	
static zval **incomplete_class_get_property_ptr_ptr(zval *object, zval *member TSRMLS_DC)
{
	incomplete_class_message(E_NOTICE TSRMLS_CC);
	return &EG(error_zval_ptr);
}

static void incomplete_class_unset_property(zval *object, zval *member TSRMLS_DC)
{
	incomplete_class_message(E_NOTICE TSRMLS_CC);
}

static int incomplete_class_has_property(zval *object, zval *member, int check_empty TSRMLS_DC)
{
	incomplete_class_message(E_NOTICE TSRMLS_CC);
	return 0;
}

static union _zend_function *incomplete_class_get_method(zval *object, char *method, int method_len TSRMLS_DC) {
	incomplete_class_message(E_ERROR TSRMLS_CC);
	return NULL;
}

/* {{{ php_create_incomplete_class
 */
static zend_object_value php_create_incomplete_object(zend_class_entry *class_type TSRMLS_DC) {
	zend_object *object;
	zend_object_value value;
	
	value = zend_objects_new(&object, class_type TSRMLS_CC);
	value.handlers = &php_incomplete_object_handlers;
	
	ALLOC_HASHTABLE(object->properties);
	zend_hash_init(object->properties, 0, NULL, ZVAL_PTR_DTOR, 0);
	return value;
}

zend_class_entry *php_create_incomplete_class(TSRMLS_D)
{
	zend_class_entry incomplete_class;

	INIT_CLASS_ENTRY(incomplete_class, INCOMPLETE_CLASS, NULL);
	incomplete_class.create_object = php_create_incomplete_object;

	memcpy(&php_incomplete_object_handlers, &std_object_handlers, sizeof(zend_object_handlers));
	php_incomplete_object_handlers.read_property = incomplete_class_get_property;
	php_incomplete_object_handlers.has_property = incomplete_class_has_property;
	php_incomplete_object_handlers.unset_property = incomplete_class_unset_property;
	php_incomplete_object_handlers.write_property = incomplete_class_write_property;
	php_incomplete_object_handlers.get_property_ptr_ptr = incomplete_class_get_property_ptr_ptr;
    php_incomplete_object_handlers.get_method = incomplete_class_get_method;
	
	return zend_register_internal_class(&incomplete_class TSRMLS_CC);
}
/* }}} */

/* {{{ php_lookup_class_name
 */
char *php_lookup_class_name(zval *object, size_t *nlen)
{
	zval **val;
	char *retval = NULL;
	HashTable *object_properties;
	TSRMLS_FETCH();

	object_properties = Z_OBJPROP_P(object);

	if (zend_hash_find(object_properties, MAGIC_MEMBER, sizeof(MAGIC_MEMBER), (void **) &val) == SUCCESS) {
		retval = estrndup(Z_STRVAL_PP(val), Z_STRLEN_PP(val));

		if (nlen)
			*nlen = Z_STRLEN_PP(val);
	}

	return (retval);
}
/* }}} */

/* {{{ php_store_class_name
 */
void php_store_class_name(zval *object, const char *name, size_t len)
{
	zval *val;
	TSRMLS_FETCH();

	MAKE_STD_ZVAL(val);

	Z_TYPE_P(val)   = IS_STRING;
	Z_STRVAL_P(val) = estrndup(name, len);
	Z_STRLEN_P(val) = len;

	zend_hash_update(Z_OBJPROP_P(object), MAGIC_MEMBER, sizeof(MAGIC_MEMBER), &val, sizeof(val), NULL);
}
/* }}} */

/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 * vim600: sw=4 ts=4 fdm=marker
 * vim<600: sw=4 ts=4
 */
